from .axis import Axis
from .bars import Bars
from .confusion_matrix import ConfusionMatrix
from .curve import Curve, CurveDataVector
from .datavector import DataVector
from .figure import Figure
from .hist import Histogram
from .scatter import Scatter
